/** Note: We get the same result as Static.java. Except we have the ability to configure
		Descendent with whatever "Ancestor" instance we desire. What we lose is the ability to add
		new behaviors to Ancestor without having to modify Descendent. However, we still have the ability
		to add behavior to "Ancestor" without having to modify clients of Descendent by providing the
		appropriate Descendent wrapper methods.
		Additionally, if Descendent uses a factory to get the "Ancestor" instance, we no longer need to
		pass the "Ancestor" instance to the Descendent constructor.
*/
public class Dynamic {
	public static void main(String args[]) {
		Ancestor a = new Descendent(new ReusableImpl());
		//a.perform();
		a.strategy();
		a.reuse();
		a.template();
	}
}

abstract class Ancestor {
	private Reusable reusable;

	public Ancestor(Reusable reusable) {
		this.reusable = reusable;
	}

	//Use inheritence for polymorphism.
	public abstract void strategy();

	//Using inheritance for reuse.
	public final void reuse() { this.reusable.reuse(); }

	//Use inheritance for reuse & polymorphism.
	protected abstract void templateMethod();
	public void template() {
		System.out.println("Ancestor template");
		this.templateMethod();
	}
}

class Descendent extends Ancestor {

	public Descendent(Reusable reusable) {
		super(reusable);
	}

	protected void templateMethod() { System.out.println("Descendent templateMethod"); }

	public void strategy() { System.out.println("Descendent strategy"); }
}

interface Reusable {
	public void reuse();
}

class ReusableImpl implements Reusable {
	public void reuse() { System.out.println("Ancestor reuse"); }
}

